import typing
from typing import Optional

from HABApp.core.errors import ItemNotFoundException
from HABApp.core.internals import uses_item_registry
from HABApp.core.items import BaseValueItem
from HABApp.core.lib import hsb_to_rgb, rgb_to_hsb

HUE_FACTOR = 360
PERCENT_FACTOR = 100

item_registry = uses_item_registry()


class ColorItem(BaseValueItem):
    """Item for dealing with color related values"""

    def __init__(self, name: str, hue=0.0, saturation=0.0, brightness=0.0):
        super().__init__(name=name, initial_value=(hue, saturation, brightness))

        self.hue: float = min(max(0.0, hue), HUE_FACTOR)
        self.saturation: float = min(max(0.0, saturation), PERCENT_FACTOR)
        self.brightness: float = min(max(0.0, brightness), PERCENT_FACTOR)

    def set_value(self, hue=0.0, saturation=0.0, brightness=0.0):
        """Set the color value

        :param hue: hue (in °)
        :param saturation: saturation (in %)
        :param brightness: brightness (in %)
        """

        # map tuples to variables
        # when processing events instead of three values we get the tuple
        if isinstance(hue, tuple):
            hue, saturation, brightness = hue

        # with None we use the already set value
        self.hue = min(max(0.0, hue), HUE_FACTOR) if hue is not None else self.hue
        self.saturation = min(max(0.0, saturation), PERCENT_FACTOR) if saturation is not None else self.saturation
        self.brightness = min(max(0.0, brightness), PERCENT_FACTOR) if brightness is not None else self.brightness

        return super().set_value(new_value=(self.hue, self.saturation, self.brightness))

    def post_value(self, hue=0.0, saturation=0.0, brightness=0.0):
        """Set a new value and post appropriate events on the HABApp event bus
        (``ValueUpdateEvent``, ``ValueChangeEvent``)

        :param hue: hue (in °)
        :param saturation: saturation (in %)
        :param brightness: brightness (in %)
        """
        super().post_value(
            # encapsulate in tuple !
            (hue if hue is not None else self.hue,
             saturation if saturation is not None else self.saturation,
             brightness if brightness is not None else self.brightness)
        )

    def get_rgb(self, max_rgb_value=255) -> typing.Tuple[int, int, int]:
        """Return a rgb equivalent of the color

        :param max_rgb_value: the max value for rgb, typically 255 (default) or 65.536
        :return: rgb tuple
        """
        return hsb_to_rgb(self.hue, self.saturation, self.brightness, max_rgb_value=max_rgb_value)

    def set_rgb(self, r, g, b, max_rgb_value=255, ndigits: Optional[int] = 2) -> 'ColorItem':
        """Set a rgb value

        :param r: red value
        :param g: green value
        :param b: blue value
        :param max_rgb_value: the max value for rgb, typically 255 (default) or 65.536
        :param ndigits: Round the hsb values to the specified digits, None to disable rounding
        :return: self
        """
        h, s, b = rgb_to_hsb(r, g, b, max_rgb_value=max_rgb_value, ndigits=ndigits)
        self.set_value(h, s, b)
        return self

    def post_rgb(self, r, g, b, max_rgb_value=255) -> 'ColorItem':
        """Set a new rgb value and post appropriate events on the HABApp event bus
        (``ValueUpdateEvent``, ``ValueChangeEvent``)

        :param r: red value
        :param g: green value
        :param b: blue value
        :param max_rgb_value: the max value for rgb, typically 255 (default) or 65.536
        :return: self
        """
        self.set_rgb(r, g, b, max_rgb_value=max_rgb_value)
        self.post_value(self.hue, self.saturation, self.brightness)
        return self

    def is_on(self) -> bool:
        """Return true if item is on"""
        return self.brightness > 0

    def is_off(self) -> bool:
        """Return true if item is off"""
        return self.brightness <= 0

    def __repr__(self):
        return f'<Color hue: {self.hue}°, saturation: {self.saturation}%, brightness: {self.brightness}%>'

    @classmethod
    def get_create_item(cls, name: str, hue=0.0, saturation=0.0, brightness=0.0):
        """Creates a new item in HABApp and returns it or returns the already existing one with the given name

        :param name: item name
        :param initial_value: state the item will have if it gets created
        :return: item
        """
        assert isinstance(name, str), type(name)

        try:
            item = item_registry.get_item(name)
        except ItemNotFoundException:
            item = item_registry.add_item(cls(name, hue=hue, saturation=saturation, brightness=brightness))

        assert isinstance(item, cls), f'{cls} != {type(item)}'
        return item



from HABApp.core.const.hints import HasNameAttr as _HasNameAttr


class HABAppException(Exception):
    pass


class ProxyObjHasNotBeenReplacedError(HABAppException):
    def __init__(self, obj) -> None:
        super().__init__(f'{obj} has not been replaced on startup!')


class ItemNotFoundException(HABAppException):
    def __init__(self, name: str):
        super().__init__(f'Item {name} does not exist!')
        self.name: str = name


class ItemAlreadyExistsError(HABAppException):
    def __init__(self, name: str):
        super().__init__(f'Item {name} does already exist and can not be added again!')
        self.name: str = name


class ContextNotFoundError(HABAppException):
    pass


class ContextNotSetError(HABAppException):
    pass


class ContextBoundObjectIsAlreadyLinkedError(HABAppException):
    pass


class ContextBoundObjectIsAlreadyUnlinkedError(HABAppException):
    pass


# ----------------------------------------------------------------------------------------------------------------------
# Value errors
# ----------------------------------------------------------------------------------------------------------------------
class HABAppValueError(ValueError, HABAppException):
    pass


class ItemValueIsNoneError(HABAppValueError):
    @classmethod
    def from_item(cls, item: _HasNameAttr):
        return cls(f'Item value is None (item "{item.name:s}")')


class InvalidItemValue(HABAppValueError):
    @classmethod
    def from_item(cls, item: _HasNameAttr, value):
        return cls(f'Invalid value for {item.__class__.__name__} {item.name:s}: {value}')



from __future__ import annotations

from threading import Lock

from .limiter import Limiter


LOCK = Lock()

_LIMITERS: dict[str, Limiter] = {}


def RateLimiter(name: str) -> Limiter:
    """Create a new rate limiter or return an already existing one with a given name.

    :param name: case insensitive name of limiter
    :return: Rate limiter object
    """

    key = name.lower()

    with LOCK:
        if (obj := _LIMITERS.get(key)) is None:
            _LIMITERS[key] = obj = Limiter(name)

    return obj



import gzip
import shutil
from datetime import date, datetime
from logging.handlers import RotatingFileHandler
from pathlib import Path


class MidnightRotatingFileHandler(RotatingFileHandler):
    """A rotating file handler that checks once after midnight if the configured size has been exceeded and
    then rotates the file
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.last_check: date = datetime.now().date()

    def shouldRollover(self, record):
        date = datetime.now().date()
        if date == self.last_check:
            return 0
        self.last_check = date
        return super().shouldRollover(record)


class CompressedMidnightRotatingFileHandler(MidnightRotatingFileHandler):
    """Same as ``MidnightRotatingFileHandler`` but rotates the file to a gzipped archive (``.gz``)

    """

    def __init__(self, *args, **kwargs):
        self.namer = self.compressed_namer
        self.rotator = self.compressed_rotator
        super().__init__(*args, **kwargs)

    def compressed_namer(self, default_name: str) -> str:
        return default_name + ".gz"

    def compressed_rotator(self, source: str, dest: str):
        src = Path(source)
        with src.open('rb') as f_in, gzip.open(dest, 'wb') as f_out:
            shutil.copyfileobj(f_in, f_out)
        src.unlink()



from .threshold import Threshold
from .statistics import Statistics
from .listener_groups import EventListenerGroup
from .fade import Fade
from .rate_limiter import RateLimiter

from . import functions
from . import multimode



from typing import TYPE_CHECKING, Final, FrozenSet, Mapping, Optional, Tuple

from HABApp.core.errors import ItemValueIsNoneError, InvalidItemValue
from HABApp.openhab.definitions import OnOffValue
from HABApp.openhab.items.base_item import MetaData, OpenhabItem
from HABApp.openhab.items.commands import OnOffCommand


if TYPE_CHECKING:
    Tuple = Tuple
    Optional = Optional
    FrozenSet = FrozenSet
    Mapping = Mapping
    MetaData = MetaData


ON: Final = OnOffValue.ON
OFF: Final = OnOffValue.OFF


class SwitchItem(OpenhabItem, OnOffCommand):
    """SwitchItem which accepts and converts the data types from OpenHAB

    :ivar str name: |oh_item_desc_name|
    :ivar str value: |oh_item_desc_value|

    :ivar Optional[str] label: |oh_item_desc_label|
    :ivar FrozenSet[str] tags: |oh_item_desc_tags|
    :ivar FrozenSet[str] groups: |oh_item_desc_group|
    :ivar Mapping[str, MetaData] metadata: |oh_item_desc_metadata|
    """

    @staticmethod
    def _state_from_oh_str(state: str):
        if state != ON and state != OFF:
            raise ValueError(f'Invalid value for SwitchItem: {state}')
        return state

    def set_value(self, new_value) -> bool:

        if isinstance(new_value, OnOffValue):
            new_value = new_value.value

        if new_value not in (ON, OFF, None):
            raise InvalidItemValue.from_item(self, new_value)

        return super().set_value(new_value)

    def is_on(self) -> bool:
        """Test value against on-value"""
        return self.value == ON

    def is_off(self) -> bool:
        """Test value against off-value"""
        return self.value == OFF

    def toggle(self):
        """Toggle the switch. Turns the switch on when off or off when currently on."""
        if self.value == ON:
            self.off()
        elif self.value == OFF:
            self.on()
        elif self.value is None:
            raise ItemValueIsNoneError.from_item(self)
        else:
            raise InvalidItemValue.from_item(self, self.value)

    def __str__(self):
        return self.value

    def __eq__(self, other):
        if isinstance(other, SwitchItem):
            return self.value == other.value
        if isinstance(other, str):
            return self.value == other
        if isinstance(other, int):
            if other and self.is_on():
                return True
            if not other and self.is_off():
                return True
            return False

        return NotImplemented

    def __bool__(self):
        if self.value is None:
            raise ItemValueIsNoneError.from_item(self)
        return self.value == ON



from dataclasses import dataclass
from time import monotonic
from typing import Final

from .base import BaseRateLimit, BaseRateLimitInfo


@dataclass
class LeakyBucketLimitInfo(BaseRateLimitInfo):
    time_remaining: float  #: Time remaining until the next drop


class LeakyBucketLimit(BaseRateLimit):
    def __init__(self, allowed: int, interval: int, hits: int = 0):
        super().__init__(allowed, interval, hits)

        self.drop_interval: Final = interval / allowed
        self.next_drop: float = monotonic() + self.drop_interval

    def repr_text(self):
        return f'drop_interval={self.drop_interval:.1f}s'

    def do_test_allow(self):

        while self.next_drop <= monotonic():
            self.hits -= 1
            self.next_drop += self.drop_interval

            if self.hits <= 0:
                # out of drop interval -> reset stats
                if self.next_drop <= monotonic():
                    self.next_drop = monotonic() + self.drop_interval
                    self.skips = 0

                self.hits = 0
                break

    do_allow = do_test_allow
    do_deny = None

    def info(self) -> LeakyBucketLimitInfo:
        self.do_test_allow()

        remaining = self.next_drop - monotonic()
        if remaining <= 0:
            remaining = 0

        return LeakyBucketLimitInfo(
            time_remaining=remaining, hits=self.hits, skips=self.skips, limit=self.allowed
        )



from dataclasses import dataclass
from typing import Final, List, Literal, Tuple, Union, get_args

from HABApp.core.const.const import PYTHON_310
from HABApp.util.rate_limiter.limits import BaseRateLimit, FixedWindowElasticExpiryLimit, \
    FixedWindowElasticExpiryLimitInfo, LeakyBucketLimit, LeakyBucketLimitInfo
from HABApp.util.rate_limiter.parser import parse_limit

if PYTHON_310:
    from typing import TypeAlias
else:
    from typing_extensions import TypeAlias


_LITERAL_LEAKY_BUCKET = Literal['leaky_bucket']
_LITERAL_FIXED_WINDOW_ELASTIC_EXPIRY = Literal['fixed_window_elastic_expiry']

LIMITER_ALGORITHM_HINT: TypeAlias = Literal[_LITERAL_LEAKY_BUCKET, _LITERAL_FIXED_WINDOW_ELASTIC_EXPIRY]


def _check_arg(name: str, value, allow_0=False):
    if not isinstance(value, int) or ((value <= 0) if not allow_0 else (value < 0)):
        msg = f'Parameter {name:s} must be an int >{"=" if allow_0 else ""} 0, is {value} ({type(value)})'
        raise ValueError(msg)


class Limiter:
    def __init__(self, name: str):
        self._name: Final = name
        self._limits: Tuple[BaseRateLimit, ...] = ()
        self._skips: int = 0
        self._skips_total: int = 0

    @property
    def total_skips(self) -> int:
        """A counter to track skips which can be manually reset"""
        return self._skips_total

    def __repr__(self):
        return f'<{self.__class__.__name__} {self._name:s}>'

    def add_limit(self, allowed: int, interval: int, *,
                  initial_hits: int = 0,
                  algorithm: LIMITER_ALGORITHM_HINT = 'leaky_bucket') -> 'Limiter':
        """Add a new rate limit

        :param allowed: How many hits are allowed
        :param interval: Interval in seconds
        :param initial_hits: How many hits the limit already has when it gets initially created
        :param algorithm: Which algorithm should this limit use
        """
        _check_arg('allowed', allowed)
        _check_arg('interval', interval)
        _check_arg('hits', initial_hits, allow_0=True)
        if not initial_hits <= allowed:
            msg = f'Parameter hits must be <= parameter allowed! {initial_hits:d} <= {allowed:d}!'
            raise ValueError(msg)

        if algorithm == get_args(_LITERAL_LEAKY_BUCKET)[0]:
            cls = LeakyBucketLimit
        elif algorithm == get_args(_LITERAL_FIXED_WINDOW_ELASTIC_EXPIRY)[0]:
            cls = FixedWindowElasticExpiryLimit
        else:
            msg = f'Unknown algorithm "{algorithm}"'
            raise ValueError(msg)

        # Check if we have already added an algorithm with these parameters
        for window in self._limits:
            if isinstance(window, cls) and window.allowed == allowed and window.interval == interval:
                return self

        limit = cls(allowed, interval, hits=initial_hits)
        self._limits = tuple(sorted([*self._limits, limit], key=lambda x: x.interval))
        return self

    def parse_limits(self, *text: str,
                     initial_hits: int = 0,
                     algorithm: LIMITER_ALGORITHM_HINT = 'leaky_bucket') -> 'Limiter':
        """Add one or more limits in textual form, e.g. ``5 in 60s``, ``10 per hour`` or ``10/15 mins``.
        If the limit does already exist it will not be added again.

        :param text: textual description of limit
        :param initial_hits: How many hits the limit already has when it gets initially created
        :param algorithm: Which algorithm should these limits use
        """
        for limit in [parse_limit(t) for t in text]:
            self.add_limit(*limit, initial_hits=initial_hits, algorithm=algorithm)
        return self

    def allow(self) -> bool:
        """Test the limit(s).

        :return: ``True`` if allowed, ``False`` if forbidden
        """
        if not self._limits:
            msg = 'No limits defined!'
            raise ValueError(msg)

        clear_skipped = True

        for limit in self._limits:
            if not limit.allow():
                self._skips += 1
                self._skips_total += 1
                return False

            # allow increments hits, if it's now 1 it was 0 before
            if limit.hits != 1:
                clear_skipped = False

        if clear_skipped:
            self._skips = 0

        return True

    def test_allow(self) -> bool:
        """Test the limit(s) without hitting it. Calling this will not increase the hit counter.

        :return: ``True`` if allowed, ``False`` if forbidden
        """

        if not self._limits:
            msg = 'No limits defined!'
            raise ValueError(msg)

        clear_skipped = True

        for limit in self._limits:
            if not limit.test_allow():
                return False

            if limit.hits != 0:
                clear_skipped = False

        if clear_skipped:
            self._skips = 0
        return True

    def info(self) -> 'LimiterInfo':
        """Get some info about the limiter and the defined windows
        """

        return LimiterInfo(
            skips=self._skips, total_skips=self._skips_total,
            limits=[limit.info() for limit in self._limits]
        )

    def reset(self) -> 'Limiter':
        """Reset the skip counter"""
        self._skips_total = 0
        return self


@dataclass
class LimiterInfo:
    skips: int          #: How many entries were skipped in the active interval(s)
    total_skips: int    #: How many entries were skipped in total
    limits: List[Union[FixedWindowElasticExpiryLimitInfo, LeakyBucketLimitInfo]]    #: Info for every limit



from dataclasses import dataclass
from typing import Final


@dataclass
class BaseRateLimitInfo:
    hits: int               #: Hits
    skips: int              #: Skips
    limit: int              #: Boundary

    @property
    def hits_remaining(self) -> int:
        return self.limit - self.hits


class BaseRateLimit:
    def __init__(self, allowed: int, interval: int, hits: int = 0):
        super().__init__()
        assert allowed > 0, allowed
        assert interval > 0, interval
        assert 0 <= hits <= allowed

        self.interval: Final = interval
        self.allowed: Final = allowed

        self.hits: int = hits
        self.skips: int = 0

    def repr_text(self) -> str:
        return ''

    def __repr__(self):
        return (
            f'<{self.__class__.__name__} hits={self.hits:d}/{self.allowed:d} interval={self.interval:d}s '
            f'{self.repr_text():s}>'
        )

    def do_test_allow(self):
        raise NotImplementedError()

    def do_allow(self):
        raise NotImplementedError()

    def do_deny(self):
        raise NotImplementedError()

    def info(self) -> BaseRateLimitInfo:
        raise NotImplementedError()

    def allow(self, weight: int = 1) -> bool:
        if not isinstance(weight, int) or weight <= 0:
            msg = f'weight must be an int > 0, is {weight}'
            raise ValueError(msg)

        self.do_allow()

        self.hits += weight
        if self.hits <= self.allowed:
            return True

        self.skips += 1
        self.hits = self.allowed

        if self.do_deny:
            self.do_deny()
        return False

    def test_allow(self, weight: int = 1) -> bool:
        if not isinstance(weight, int) or weight <= 0:
            msg = f'weight must be an int > 0, is {weight}'
            raise ValueError(msg)

        self.do_test_allow()
        return self.hits + weight <= self.allowed



import logging
from typing import Union

import HABApp
import HABApp.core
import HABApp.openhab.events
from HABApp.core.asyncio import create_task_from_async
from HABApp.core.errors import ItemNotFoundException
from HABApp.core.events import ValueChangeEvent, ValueUpdateEvent
from HABApp.core.internals import uses_get_item, uses_post_event
from HABApp.core.logger import log_warning
from HABApp.core.wrapper import process_exception
from HABApp.openhab.definitions.topics import TOPIC_ITEMS, TOPIC_THINGS
from HABApp.openhab.events import (
    GroupStateChangedEvent,
    GroupStateUpdatedEvent,
    ItemAddedEvent,
    ItemRemovedEvent,
    ItemUpdatedEvent,
    ThingAddedEvent,
    ThingConfigStatusInfoEvent,
    ThingRemovedEvent,
    ThingStatusInfoEvent,
    ThingUpdatedEvent,
)
from HABApp.openhab.item_to_reg import (
    add_thing_to_registry,
    add_to_registry,
    remove_from_registry,
    remove_thing_from_registry,
)
from HABApp.openhab.map_events import get_event


log = logging.getLogger('HABApp.openhab.items')

post_event = uses_post_event()
get_item = uses_get_item()


def on_sse_event(event_dict: dict, oh_3: bool):
    try:
        # Lookup corresponding OpenHAB event
        event = get_event(event_dict)

        # Update item in registry BEFORE posting to the event bus
        # so the items have the correct state when we process the event in a rule
        try:
            if isinstance(event, ValueUpdateEvent):
                __item = get_item(event.name)  # type: HABApp.core.items.base_valueitem.BaseValueItem
                __item.set_value(event.value)
                post_event(event.name, event)
                return None

            if isinstance(event, ValueChangeEvent):
                # Workaround because there is no GroupItemStateEvent in OH3
                if oh_3 and isinstance(event, GroupStateChangedEvent):
                    __item = get_item(event.name)  # type: HABApp.openhab.items.GroupItem
                    __item.set_value(event.value)
                    # Manually issue an GroupStateUpdatedEvent so we are consistent with OH4 behavior
                    post_event(event.name, GroupStateUpdatedEvent(event.name, event.item, event.value))
                post_event(event.name, event)
                return None

            if isinstance(event, (ThingStatusInfoEvent, ThingUpdatedEvent, ThingConfigStatusInfoEvent)):
                __thing = get_item(event.name)   # type: HABApp.openhab.items.Thing
                __thing.process_event(event)
                post_event(event.name, event)
                return None

        except ItemNotFoundException:
            log_warning(log, f'Received {event.__class__.__name__} for {event.name} but item does not exist!')

            # Post the event anyway
            post_event(event.name, event)
            return None

        # Events that add items to the item registry
        # These events require that we query openHAB because of the metadata, so we have to do it in a task
        if isinstance(event, (ItemAddedEvent, ItemUpdatedEvent)):
            create_task_from_async(item_event(event))
            return None

        # Events that remove items from the item registry
        if isinstance(event, ItemRemovedEvent):
            remove_from_registry(event.name)
            post_event(TOPIC_ITEMS, event)
            return None

        # Events that add things to the item registry
        if isinstance(event, ThingAddedEvent):
            add_thing_to_registry(event)
            post_event(TOPIC_THINGS, event)
            return None

        # Events that remove things from the item registry
        if isinstance(event, ThingRemovedEvent):
            remove_thing_from_registry(event.name)
            post_event(TOPIC_THINGS, event)
            return None

        # Unknown Event -> just forward it to the event bus
        post_event(event.name, event)
    except Exception as e:
        process_exception(func=on_sse_event, e=e)
        return None


async def item_event(event: Union[ItemAddedEvent, ItemUpdatedEvent]):
    try:
        from HABApp.openhab.map_items import map_item

        name = event.name

        # Since metadata is not part of the event we have to request it through the item
        if (cfg := await HABApp.openhab.interface_async.async_get_item(name)) is None:
            return None

        new_item = map_item(name, event.type, None, event.label, event.tags, event.groups, metadata=cfg.metadata)
        if new_item is None:
            return None

        add_to_registry(new_item)
        # Send Event to Event Bus
        post_event(TOPIC_ITEMS, event)
        return None
    except Exception as e:
        process_exception(func=item_event, e=e)
        return None



from dataclasses import dataclass
from time import monotonic

from .base import BaseRateLimit, BaseRateLimitInfo


@dataclass
class FixedWindowElasticExpiryLimitInfo(BaseRateLimitInfo):
    time_remaining: float   #: Time remaining until this window will reset


class FixedWindowElasticExpiryLimit(BaseRateLimit):
    def __init__(self, allowed: int, interval: int, hits: int = 0):
        super().__init__(allowed, interval, hits)

        self.start: float = -1.0
        self.stop: float = -1.0

    def repr_text(self):
        return f'window={self.stop - self.start:.0f}s'

    def do_test_allow(self):
        if self.stop <= monotonic():
            self.hits = 0
            self.skips = 0

    def do_allow(self):
        now = monotonic()

        if self.stop <= now:
            self.hits = 0
            self.skips = 0
            self.start = now
            self.stop = now + self.interval

    def do_deny(self):
        self.stop = monotonic() + self.interval

    def info(self) -> FixedWindowElasticExpiryLimitInfo:
        self.do_test_allow()

        remaining = self.stop - monotonic()
        if remaining <= 0:
            remaining = 0

        if not remaining and not self.hits:
            remaining = self.interval

        return FixedWindowElasticExpiryLimitInfo(
            time_remaining=remaining, hits=self.hits, skips=self.skips, limit=self.allowed
        )



import re
from typing import Tuple

LIMIT_REGEX = re.compile(
    r"""
    \s* ([1-9][0-9]*)
    \s* (/|per|in)
    \s* ([1-9][0-9]*)?
    \s* (s|sec|second|m|min|minute|h|hour|day|month|year)s?
    \s*""",
    re.IGNORECASE | re.VERBOSE,
)


def parse_limit(text: str) -> Tuple[int, int]:
    if not isinstance(text, str) or not (m := LIMIT_REGEX.fullmatch(text)):
        msg = f'Invalid limit string: "{text:s}"'
        raise ValueError(msg)

    count, per, factor, interval = m.groups()

    interval_secs = {
        's': 1, 'sec': 1, 'second': 1, 'm': 60, 'min': 60, 'minute': 60, 'hour': 3600, 'h': 3600,
        'day': 24 * 3600, 'month': 30 * 24 * 3600, 'year': 365 * 24 * 3600
    }[interval]

    return int(count), int(1 if factor is None else factor) * interval_secs
